<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

require_once(APPPATH . '/models/expenses/mod.expenses_pmf.php');

class Expenses_admin extends Expenses_pmf 
{
	public	$options	= array();

	private $_user_id	= 0;

	function __construct()
	{
		parent::__construct();
		$this->_user_id	= $this->ot_user->get('user_id');
	}
	
	public function expenses_list_query($select='', $options=array()) 
	{
		$this->db->select($select, false);
		$this->db->from('project as pr');
		$this->db->join('project_expense as ex', 'pr.entity_id=ex.entity_id AND pr.project_id=ex.project_id', 'INNER');
		$this->db->join('expense_type as ext', 'ex.entity_id=ext.entity_id AND ex.expense_type_id=ext.expense_type_id', 'LEFT');
		$this->db->join('entity_users as u', 'u.entity_id=ex.entity_id AND u.user_id=ex.created_by', 'INNER');
		
		
		$where	= array('pr.entity_id' => $this->ot_app->get('entity_id'));
		if (!empty($options['filters']))
		{
			foreach($options['filters'] as $key=>$val) {
				if (is_array($val))
					$this->db->where_in($key,$val);
				else
					$where[$key] = $val;
			}
		}
		
		$month_dates = array();
		$this->db->where($where);
		if (isset($options['start_date']))
		{
			$month_dates['start_date']	= $options['start_date'];
			if (isset($options['finish_date'])) {
				$month_dates['end_date']	= $options['finish_date'];
				$this->db->where('ex.expense_date BETWEEN "' .$month_dates['start_date']. '" AND "' . $month_dates['end_date'] . '"', NULL, FALSE);
			} else {
				$this->db->where('ex.expense_date',$month_dates['start_date'],NULL,FALSE);
			} 	
		} elseif (!empty($options['expense_month'])) {
			$month_dates = $this->get_start_end_dates(!empty($options['expense_month'])?$options['expense_month']:'');
			$this->db->where('ex.expense_date BETWEEN "' .$month_dates['start_date']. '" AND "' . $month_dates['end_date'] . '"', NULL, FALSE);
		}
		
		$order_by 			= 'ex.expense_date';
		$order_by_direction = 'desc';

		$this->db->order_by($order_by, $order_by_direction);

		return $this->db->get();		
	}
	
	public function get_expenses($select='', $options=array()) 
	{
		$select	= empty($select) ? 'ex.expense_id, ex.project_id, ex.amount, ex.created_by, DATE_FORMAT(ex.expense_date, "%Y-%m-%d") as expense_date, ex.notes, ex.expense_type_id, ex.approved_by, ex.invoice_id, ex.quantity, ex.unit_price, pr.project_name, pr.project_code,  u.first_name, u.user_image, ext.name, ext.units' : $select;
		if (empty($options))
		{
			$options	= array_merge($options, $this->options);
		}
		
		$options['filters']['ex.approved_by >'] = '0';

		$query = $this->expenses_list_query($select, $options);

		$result = $query->result_array();
		
		foreach($result as $key=>$res) 
		{
			$result[$key]['edit_permission'] 		= 1;
			$result[$key]['proj_edit_permission'] 	= 0;
			if($res['created_by'] == $this->_user_id)
			{
				$result[$key]['proj_edit_permission'] 	= 1;
			}
		}
		return $result;
	}

	public function get_approve_expenses($select='', $options=array()) 
	{
		$select	= empty($select) ? 'ex.expense_id, ex.project_id, ex.amount, ex.expense_date, ex.notes, ex.expense_type_id, ex.approved_by, ex.invoice_id, ex.quantity, ex.unit_price, pr.project_name, pr.project_code,  u.first_name, u.user_image, ext.name, ext.units' : $select;
		if (empty($options))
		{
			$options	= array_merge($options, $this->options);
		}
		
		$options['filters']['ex.approved_by'] = '0';
		
		$query = $this->expenses_list_query($select, $options);

		return $query->result_array();
	}
	
	public function get_projects() 
	{
		$this->db->distinct();
		$this->db->select('pr.project_id, pr.project_code, concat(pr.project_code, pr.project_name) as project_name', false);
		$this->db->from('project pr');
		$this->db->join('project_users pru', 'pr.project_id = pru.project_id','INNER');

		$this->db->where(array('pr.entity_id'=>$this->ot_app->get('entity_id'),'pr.is_active'=>1));
		$this->db->order_by('pr.project_name');
		
		$query = $this->db->get();
		//echo $this->db->last_query();
		$result = $query->result_array(); 
		return $result;
	}
	
	public function delete ($id)
	{
		$where = array('expense_id ' => $id, 'entity_id'=>$this->ot_app->get('entity_id'));
		$this->db->where($where);
		if ($this->db->delete('project_expense') &&  $this->db->affected_rows())
		{
			$where = array('module_id ' => $id, 'module_type' => 'E');
			$this->db->where($where);
			$this->db->delete('attachment');
			
			return true;
		}
				
		return false;
	}
}
